print("import: 'azure'")
import azure

print("import: 'azure.common'")
import azure.common

print("import: 'azure.mgmt'")
import azure.mgmt

print("import: 'azure.servicebus'")
import azure.servicebus

print("import: 'azure.servicemanagement'")
import azure.servicemanagement

